#ifndef TD3DClipStatusH
#define TD3DClipStatusH

// ==========================================================================
// File:         TD3DClipStatus.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TD3DClipStatus Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"

// ==========================================================================
// Set:         TD3DClipStatusFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TD3DClipStatus::Flags property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Flags property description below. 
// ==========================================================================
enum TD3DClipStatus_Flags_Flags
    {
    d3dclipstatus_extents2,
    d3dclipstatus_extents3,
    d3dclipstatus_status
    };
typedef Set< TD3DClipStatus_Flags_Flags, 
             d3dclipstatus_extents2, 
             d3dclipstatus_status >  TD3DClipStatus_Flags_Set;
// ==========================================================================
// Set:         TD3DClipStatusStatus_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TD3DClipStatus::Status property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Status property description below. 
// ==========================================================================
enum TD3DClipStatus_Status_Flags
    {
    d3dclip_back,
    d3dclip_bottom,
    d3dclip_front,
    d3dclip_gen0,
    d3dclip_gen1,
    d3dclip_gen2,
    d3dclip_gen5,
    d3dclip_left,
    d3dclip_right,
    d3dclip_top,
    d3dstatus_clipintersectionall,
    d3dstatus_clipintersectionback,
    d3dstatus_clipintersectionbottom,
    d3dstatus_clipintersectionfront,
    d3dstatus_clipintersectiongen0,
    d3dstatus_clipintersectiongen1,
    d3dstatus_clipintersectiongen2,
    d3dstatus_clipintersectiongen3,
    d3dstatus_clipintersectionleft,
    d3dstatus_clipintersectionright,
    d3dstatus_clipintersectiontop,
    d3dstatus_clipunionall,
    d3dstatus_clipunionback,
    d3dstatus_clipunionbottom,
    d3dstatus_clipunionfront,
    d3dstatus_clipuniongen0,
    d3dstatus_clipuniongen1,
    d3dstatus_clipunionleft,
    d3dstatus_clipunionright,
    d3dstatus_clipuniontop,
    d3dstatus_default,
    d3dstatus_znotvisible
    };
typedef Set< TD3DClipStatus_Status_Flags, 
             d3dclip_back, 
             d3dstatus_znotvisible >  TD3DClipStatus_Status_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DClipStatus
// Description: The TD3DClipStatus component wraps the DirectX D3DCLIPSTATUS 
//              structure which describes the current status and extent of 
//              the clipping region.   
//              
//              This component is used by the TDx_3DDevice::GetClipStatus() 
//              and TDx_3DDevice::SetClipStatus() methods to define the 
//              clipping region or status for a 2 or 3 dimensional space. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DClipStatus : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DClipStatus : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines the type of information the 
    //              component is holding. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TD3DClipStatus_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Status_
    // Description: The Status property defines the current clip status and 
    //              which clipping planes are active. 
    //              The described effect applies when the flag is set. 
    //              
    //              By default, both D3DSTATUS_CLIPINTERSECTIONALL and 
    //              D3DSTATUS_ZNOTVISIBLE are set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Status' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Status 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TD3DClipStatus_Status_Set Status_ = { read=FGetStatus_, 
    write=FSetStatus_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    MaxX
    // Description: The MaxX property defines the maximum X axis value of the 
    //              current clipping region. 
    // ----------------------------------------------------------------------
    __property float MaxX = { read=FGetMaxX, write=FSetMaxX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxY
    // Description: The MaxY property defines the maximum Y axis value of the 
    //              current clipping region. 
    // ----------------------------------------------------------------------
    __property float MaxY = { read=FGetMaxY, write=FSetMaxY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxZ
    // Description: The MaxZ property defines the maximum Z axis value of the 
    //              current clipping region. 
    // ----------------------------------------------------------------------
    __property float MaxZ = { read=FGetMaxZ, write=FSetMaxZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MinX
    // Description: The MinX property defines the minimum X axis value of the 
    //              current clipping region. 
    // ----------------------------------------------------------------------
    __property float MinX = { read=FGetMinX, write=FSetMinX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MinY
    // Description: The MinY property defines the minimum Y axis value of the 
    //              current clipping region. 
    // ----------------------------------------------------------------------
    __property float MinY = { read=FGetMinY, write=FSetMinY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MinZ
    // Description: The MinZ property defines the minimum Z axis value of the 
    //              current clipping region. 
    // ----------------------------------------------------------------------
    __property float MinZ = { read=FGetMinZ, write=FSetMinZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DClipStatus component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DClipStatus method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines the type of information the 
    //              component is holding. 
    //              The described effect applies when the flag is set. 
    // Flags:       D3DCLIPSTATUS_EXTENTS2 -
    //                The component is describing the current extent, in two 
    //                dimensions, of the clipping region. 
    //                
    //                The MaxX, MaxY, MinX and MinY properties will contain 
    //                valid information. 
    //                
    //                Both this flag and the D3DCLIPSTATUS_EXTENTS3 flag 
    //                cannot be set at the same time, they are mutually 
    //                exclusive. 
    //              D3DCLIPSTATUS_EXTENTS3 -
    //                The component is describing the current extent, in 
    //                three dimensions, of the clipping region. 
    //                
    //                The MaxX, MaxY, MaxZ, MinX, MinY and MinZ properties 
    //                will contain valid information. 
    //                
    //                Both this flag and the D3DCLIPSTATUS_EXTENTS2 flag 
    //                cannot be set at the same time, they are mutually 
    //                exclusive. 
    //                
    //                Note: This flag was not yet implemented by Microsoft in 
    //                DirectX 7. 
    //              D3DCLIPSTATUS_STATUS -
    //                The Status property of this component will contain 
    //                valid information. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal D3DCLIPSTATUS structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Status
    // Description: The Status property defines the current clip status and 
    //              which clipping planes are active. 
    //              The described effect applies when the flag is set. 
    //              
    //              By default, both D3DSTATUS_CLIPINTERSECTIONALL and 
    //              D3DSTATUS_ZNOTVISIBLE are set. 
    // Flags:       D3DCLIP_BACK -
    //                All vertices are clipped by the back clipping plane of 
    //                the viewing frustrum. 
    //                This plane is delimited by the points (MinX, MinY, 
    //                MaxZ), (MaxX, MinY, MaxZ), (MaxX, MinY, MaxZ) and 
    //                (MaxX, MaxY, MaxZ). 
    //              D3DCLIP_BOTTOM -
    //                All vertices are clipped by the bottom clipping plane 
    //                of the viewing frustum. 
    //                This plane is delimited by the points (MinX, MinY, 
    //                MinZ), (MaxX, MinY, MinZ), (MinX, MinY, MaxZ) and 
    //                (MaxX, MinY, MaxZ). 
    //              D3DCLIP_FRONT -
    //                All vertices are clipped by the front clipping plane of 
    //                the viewing frustum. 
    //                This plane is delimited by the points (MinX, MinY, 
    //                MinZ), (MaxX, MinY, MinZ), (MaxX, MinY, MinZ) and 
    //                (MaxX, MaxY, MinZ). 
    //              D3DCLIP_GEN0 -
    //                A clipping plane that is defined by the application. 
    //              D3DCLIP_GEN1 -
    //                A clipping plane that is defined by the application. 
    //              D3DCLIP_GEN2 -
    //                A clipping plane that is defined by the application. 
    //              D3DCLIP_GEN3 -
    //                A clipping plane that is defined by the application. 
    //              D3DCLIP_GEN4 -
    //                A clipping plane that is defined by the application. 
    //              D3DCLIP_GEN5 -
    //                A clipping plane that is defined by the application. 
    //              D3DCLIP_LEFT -
    //                All vertices are clipped by the left clipping plane of 
    //                the viewing frustum. 
    //                This plane is delimited by the points (MinX, MinY, 
    //                MinZ), (MinX, MaxY, MinZ), (MinX, MinY, MaxZ) and 
    //                (MinX, MaxY, MaxZ). 
    //              D3DCLIP_RIGHT -
    //                All vertices are clipped by the right clipping plane of 
    //                the viewing frustum. 
    //                This plane is delimited by the points (MaxX, MinY, 
    //                MinZ), (MaxX, MaxY, MinZ), (MaxX, MinY, MaxZ) and 
    //                (MaxX, MaxY, MaxZ). 
    //              D3DCLIP_TOP -
    //                All vertices are clipped by the top clipping plane of 
    //                the viewing frustum. 
    //                This plane is delimited by the points (MinX, MaxY, 
    //                MinZ), (MaxX, MaxY, MinZ), (MinX, MaxY, MaxZ) and 
    //                (MaxX, MaxY, MaxZ). 
    //              D3DSTATUS_CLIPINTERSECTIONALL -
    //                Equivilent to setting all the clip intersection flags. 
    //                This flag plus D3DSTATUS_ZNOTVISIBLE form the default 
    //                setting. 
    //              D3DSTATUS_CLIPINTERSECTIONBACK -
    //                Compares (logical AND) the back clipping plane with the 
    //                clip flags of the vertices. 
    //              D3DSTATUS_CLIPINTERSECTIONBOTTOM -
    //                Compares (logical AND) the bottom clipping plane with 
    //                the clip flags of the vertices. 
    //              D3DSTATUS_CLIPINTERSECTIONFRONT -
    //                Compares (logical AND) the front clipping plane with 
    //                the clip flags of the vertices. 
    //              D3DSTATUS_CLIPINTERSECTIONGEN0 -
    //                Compares (logical AND) an application defined clipping 
    //                plane with the clip flags of the vertices. 
    //              D3DSTATUS_CLIPINTERSECTIONGEN1 -
    //                Compares (logical AND) an application defined clipping 
    //                plane with the clip flags of the vertices. 
    //              D3DSTATUS_CLIPINTERSECTIONGEN2 -
    //                Compares (logical AND) an application defined clipping 
    //                plane with the clip flags of the vertices. 
    //              D3DSTATUS_CLIPINTERSECTIONGEN3 -
    //                Compares (logical AND) an application defined clipping 
    //                plane with the clip flags of the vertices. 
    //              D3DSTATUS_CLIPINTERSECTIONGEN4 -
    //                Compares (logical AND) an application defined clipping 
    //                plane with the clip flags of the vertices. 
    //              D3DSTATUS_CLIPINTERSECTIONGEN5 -
    //                Compares (logical AND) an application defined clipping 
    //                plane with the clip flags of the vertices. 
    //              D3DSTATUS_CLIPINTERSECTIONLEFT -
    //                Compares (logical AND) the left clipping plane with the 
    //                clip flags of the vertices. 
    //              D3DSTATUS_CLIPINTERSECTIONRIGHT -
    //                Compares (logical AND) the right clipping plane with 
    //                the clip flags of the vertices. 
    //              D3DSTATUS_CLIPINTERSECTIONTOP -
    //                Compares (logical AND) the top clipping plane with the 
    //                clip flags of the vertices. 
    //              D3DSTATUS_CLIPUNIONALL -
    //                Equivilent to setting all the clip union flags. 
    //                Since the CLIPUNION flags are equivilent to the basic 
    //                flags (top,bottom,left, right,back,front, gen0-gen5), 
    //                this flag can be used to essentially set all the basic 
    //                flags at once. 
    //              D3DSTATUS_CLIPUNIONBACK -
    //                All vertices are clipped by the back clipping plane of 
    //                the viewing frustum. 
    //                Equivilent to D3DCLIP_BACK. 
    //              D3DSTATUS_CLIPUNIONBOTTOM -
    //                All vertices are clipped by the bottom clipping plane 
    //                of the viewing frustum. 
    //                Equivilent to D3DCLIP_BOTTOM. 
    //              D3DSTATUS_CLIPUNIONFRONT -
    //                All vertices are clipped by the front clipping plane of 
    //                the viewing frustum. 
    //                Equivilent to D3DCLIP_FRONT. 
    //              D3DSTATUS_CLIPUNIONGEN0 -
    //                All vertices are clipped by an application defined 
    //                clipping plane. 
    //                Equivilent to D3DCLIP_GEN0. 
    //              D3DSTATUS_CLIPUNIONGEN1 -
    //                All vertices are clipped by an application defined 
    //                clipping plane. 
    //                Equivilent to D3DCLIP_GEN0. 
    //              D3DSTATUS_CLIPUNIONGEN2 -
    //                All vertices are clipped by an application defined 
    //                clipping plane. 
    //                Equivilent to D3DCLIP_GEN0. 
    //              D3DSTATUS_CLIPUNIONGEN3 -
    //                All vertices are clipped by an application defined 
    //                clipping plane. 
    //                Equivilent to D3DCLIP_GEN0. 
    //              D3DSTATUS_CLIPUNIONGEN4 -
    //                All vertices are clipped by an application defined 
    //                clipping plane. 
    //                Equivilent to D3DCLIP_GEN0. 
    //              D3DSTATUS_CLIPUNIONGEN5 -
    //                All vertices are clipped by an application defined 
    //                clipping plane. 
    //                Equivilent to D3DCLIP_GEN0. 
    //              D3DSTATUS_CLIPUNIONLEFT -
    //                All vertices are clipped by the left clipping plane of 
    //                the viewing frustum. 
    //                Equivilent to D3DCLIP_LEFT. 
    //              D3DSTATUS_CLIPUNIONRIGHT -
    //                All vertices are clipped by the right clipping plane of 
    //                the viewing frustum. 
    //                Equivilent to D3DCLIP_RIGHT. 
    //              D3DSTATUS_CLIPUNIONTOP -
    //                All vertices are clipped by the top clipping plane of 
    //                the viewing frustum. 
    //                Equivilent to D3DCLIP_TOP. 
    //              D3DSTATUS_DEFAULT -
    //                Equivilent to setting D3DSTATUS_CLIPINTERSECTIONALL and 
    //                D3DSTATUS_ZNOTVISIBLE. 
    //                This is the default setting. 
    //              D3DSTATUS_ZNOTVISIBLE -
    //                The primative is not visible. 
    //                The system sets or clears this flag when z-checking is 
    //                enabled. 
    // ----------------------------------------------------------------------
    __property dword Status = { read=FGetStatus, write=FSetStatus, default=0 
    }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DClipStatus's internal D3DCLIPSTATUS structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Property:    Status_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Status property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Status_Strings = { read=FGetStatus_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DClipStatus(TComponent* Owner); 
    virtual __fastcall ~TD3DClipStatus(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DCLIPSTATUS* Internal_D3DCLIPSTATUS_Ptr = { 
    read=FGetInternal_D3DCLIPSTATUS_Ptr, nodefault }; 

    void __fastcall Internal_D3DCLIPSTATUS_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TD3DClipStatus_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TD3DClipStatus_Flags_Set pFlags_ ); 
    TD3DClipStatus_Status_Set __fastcall FGetStatus_(); 
    void __fastcall FSetStatus_( TD3DClipStatus_Status_Set pStatus_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    float __fastcall FGetMaxX(); 
    void __fastcall FSetMaxX( float pMaxX ); 
    float __fastcall FGetMaxY(); 
    void __fastcall FSetMaxY( float pMaxY ); 
    float __fastcall FGetMaxZ(); 
    void __fastcall FSetMaxZ( float pMaxZ ); 
    float __fastcall FGetMinX(); 
    void __fastcall FSetMinX( float pMinX ); 
    float __fastcall FGetMinY(); 
    void __fastcall FSetMinY( float pMinY ); 
    float __fastcall FGetMinZ(); 
    void __fastcall FSetMinZ( float pMinZ ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetStatus(); 
    void __fastcall FSetStatus( dword pStatus ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings();

    TStringList* __fastcall FGetStatus_Strings();
// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DCLIPSTATUS* __fastcall FGetInternal_D3DCLIPSTATUS_Ptr(); 
    void __fastcall FSetInternal_D3DCLIPSTATUS( D3DCLIPSTATUS* pD3DCLIPSTATUS 
    ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;
    TStringList* fStatus_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DCLIPSTATUS fD3DCLIPSTATUS; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
