#ifndef TD3DColorValueH
#define TD3DColorValueH

// ==========================================================================
// File:         TD3DColorValue.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TD3DColorValue Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DColorValue
// Description: The TD3DColorValue component wraps the DirectX D3DCOLORVALUE 
//              structure and each property contains a value indicating the 
//              intensity of the respective color. 
//              
//              This component is used by the TD3DLight and TD3DMaterial 
//              components to hold color information. 
//              The color values usually range between 0 (black) and 1, but 
//              higher and lower values can be set to produce unusual 
//              lighting effects such as light absorbing objects or a washed 
//              out scene. 
//              
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DColorValue : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DColorValue : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DColorValue component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    A
    // Description: The A property defines the alpha component of the color. 
    //              This property is unioned with the "a" property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE A = { read=FGetA, write=FSetA, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    B
    // Description: The B property defines the blue component of the color. 
    //              This property is unioned with the "b" property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE B = { read=FGetB, write=FSetB, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DColorValue method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    G
    // Description: The G property defines the green component of the color. 
    //              This property is unioned with the "g" property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE G = { read=FGetG, write=FSetG, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    R
    // Description: The R property defines the red component of the color. 
    //              This property is unioned with the "R" property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE R = { read=FGetR, write=FSetR, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal D3DCOLORVALUE structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    a
    // Description: The a property defines the alpha component of the color. 
    //              This property is unioned with the "A" property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE a = { read=FGeta, write=FSeta, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    b
    // Description: The b property defines the blue component of the color. 
    //              This property is unioned with the "B" property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE b = { read=FGetb, write=FSetb, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    g
    // Description: The g property defines the green component of the color. 
    //              This property is unioned with the "G" property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE g = { read=FGetg, write=FSetg, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    r
    // Description: The r property defines the red component of the color. 
    //              This property is unioned with the "R" property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE r = { read=FGetr, write=FSetr, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DColorValue's internal D3DCOLORVALUE structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DColorValue(TComponent* Owner); 
    virtual __fastcall ~TD3DColorValue(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DCOLORVALUE* Internal_D3DCOLORVALUE_Ptr = { 
    read=FGetInternal_D3DCOLORVALUE_Ptr, nodefault }; 

    void __fastcall Internal_D3DCOLORVALUE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    D3DVALUE __fastcall FGetA(); 
    void __fastcall FSetA( D3DVALUE pA ); 
    D3DVALUE __fastcall FGetB(); 
    void __fastcall FSetB( D3DVALUE pB ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    D3DVALUE __fastcall FGetG(); 
    void __fastcall FSetG( D3DVALUE pG ); 
    D3DVALUE __fastcall FGetR(); 
    void __fastcall FSetR( D3DVALUE pR ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    D3DVALUE __fastcall FGeta(); 
    void __fastcall FSeta( D3DVALUE pa ); 
    D3DVALUE __fastcall FGetb(); 
    void __fastcall FSetb( D3DVALUE pb ); 
    D3DVALUE __fastcall FGetg(); 
    void __fastcall FSetg( D3DVALUE pg ); 
    D3DVALUE __fastcall FGetr(); 
    void __fastcall FSetr( D3DVALUE pr ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DCOLORVALUE* __fastcall FGetInternal_D3DCOLORVALUE_Ptr(); 
    void __fastcall FSetInternal_D3DCOLORVALUE( D3DCOLORVALUE* pD3DCOLORVALUE 
    ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DCOLORVALUE fD3DCOLORVALUE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
