#ifndef TD3DDevInfo_TexturingH
#define TD3DDevInfo_TexturingH

// ==========================================================================
// File:         TD3DDevInfo_Texturing.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TD3DDevInfo_Texturing Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DDevInfo_Texturing
// Description: The TD3DDevInfo_Texturing component wraps the DirectX 
//              D3DDEVINFO_TEXTURING structure which describes the texturing 
//              activity of the application. 
//              
//              This component is used by the TDx_3DDevice::GetInfo() method 
//              hold information on the applications texturing activity. 
//              Information relating to the number of texture surfaces 
//              created, destroyed,  loaded and locked by the current 
//              application are available. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DDevInfo_Texturing : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DDevInfo_Texturing : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ApproxBytesLoaded
    // Description: The ApproxBytesLoaded property estimates the memory, in 
    //              bytes, loaded using the TDx_3DDevice::Load() method. 
    // ----------------------------------------------------------------------
    __property dword ApproxBytesLoaded = { read=FGetApproxBytesLoaded, 
    write=FSetApproxBytesLoaded, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumCreates
    // Description: The NumCreates property defines how many textures have 
    //              been created by the current application. 
    //              
    // ----------------------------------------------------------------------
    __property dword NumCreates = { read=FGetNumCreates, 
    write=FSetNumCreates, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumDestroys
    // Description: The NumDestroys property defines how many textures have 
    //              been released by the current application. 
    // ----------------------------------------------------------------------
    __property dword NumDestroys = { read=FGetNumDestroys, 
    write=FSetNumDestroys, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumGetDCs
    // Description: The NumGetDCs property defines how many times 
    //              TDx_3DDevice::GetDC() has been called by the current 
    //              application. 
    // ----------------------------------------------------------------------
    __property dword NumGetDCs = { read=FGetNumGetDCs, write=FSetNumGetDCs, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumLoads
    // Description: The NumLoads property defines how many times 
    //              TDx_3DDevice::Load() has been called by the current 
    //              application. 
    // ----------------------------------------------------------------------
    __property dword NumLoads = { read=FGetNumLoads, write=FSetNumLoads, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumLocks
    // Description: The NumLocks property defines how many times 
    //              TDx_3DDevice::Lock() has been called by the current 
    //              application. 
    // ----------------------------------------------------------------------
    __property dword NumLocks = { read=FGetNumLocks, write=FSetNumLocks, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumPreLoads
    // Description: The NumPreLoads property defines how many times 
    //              TDx_3DDevice::PreLoad() has been called by the current 
    //              application. 
    // ----------------------------------------------------------------------
    __property dword NumPreLoads = { read=FGetNumPreLoads, 
    write=FSetNumPreLoads, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumSet
    // Description: The NumSet property defines how many times 
    //              TDx_3DDevice::SetTexture() has been called by the current 
    //              application. 
    // ----------------------------------------------------------------------
    __property dword NumSet = { read=FGetNumSet, write=FSetNumSet, default=0 
    }; 

    // ----------------------------------------------------------------------
    // Property:    NumSetLODs
    // Description: The NumSetLODs property defines how many times 
    //              TDx_3DDevice::SetLOD() has been called by the current 
    //              application. 
    // ----------------------------------------------------------------------
    __property dword NumSetLODs = { read=FGetNumSetLODs, 
    write=FSetNumSetLODs, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumSetPriorities
    // Description: The NumSetPriorities property defines how many times 
    //              TDx_3DDevice::SetPriority() has been called by the 
    //              current application. 
    // ----------------------------------------------------------------------
    __property dword NumSetPriorities = { read=FGetNumSetPriorities, 
    write=FSetNumSetPriorities, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TNewObject::Create() method after it has successfully 
    //              created the internal  used within the TNewObject 
    //              component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the  interface. 
    //              
    //              For example, you could retrieve the TNewObject 
    //              component's capabilities, Create() dependant components, 
    //              allocate memory, etc. 
    //              
    //              When used in conjunction with the TNewObject::OnDestroy() 
    //              event, you can fully automate and insulate your code on a 
    //              per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TNewObject::Destroy() method, just before the TNewObject 
    //              internal  interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the TNewObject::OnCreate() event. 
    //              For example, assume you have already setup the 
    //              TNewObject::OnCreate() event to retrieve the component's 
    //              capabilities, Create() a dependant component and allocate 
    //              some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TNewObject::OnDestroy() event to deallocate memory, 
    //              Destroy() the dependant components and, when required, 
    //              destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DDevInfo_Texturing component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DDevInfo_Texturing method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal D3DDEVINFO_TEXTURING structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DDevInfo_Texturing's internal D3DDEVINFO_TEXTURING 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DDevInfo_Texturing(TComponent* Owner); 
    virtual __fastcall ~TD3DDevInfo_Texturing(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DDEVINFO_TEXTURING* Internal_D3DDEVINFO_TEXTURING_Ptr = { 
    read=FGetInternal_D3DDEVINFO_TEXTURING_Ptr, nodefault }; 

    void __fastcall Internal_D3DDEVINFO_TEXTURING_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetApproxBytesLoaded(); 
    void __fastcall FSetApproxBytesLoaded( dword pApproxBytesLoaded ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetNumCreates(); 
    void __fastcall FSetNumCreates( dword pNumCreates ); 
    dword __fastcall FGetNumDestroys(); 
    void __fastcall FSetNumDestroys( dword pNumDestroys ); 
    dword __fastcall FGetNumGetDCs(); 
    void __fastcall FSetNumGetDCs( dword pNumGetDCs ); 
    dword __fastcall FGetNumLoads(); 
    void __fastcall FSetNumLoads( dword pNumLoads ); 
    dword __fastcall FGetNumLocks(); 
    void __fastcall FSetNumLocks( dword pNumLocks ); 
    dword __fastcall FGetNumPreLoads(); 
    void __fastcall FSetNumPreLoads( dword pNumPreLoads ); 
    dword __fastcall FGetNumSet(); 
    void __fastcall FSetNumSet( dword pNumSet ); 
    dword __fastcall FGetNumSetLODs(); 
    void __fastcall FSetNumSetLODs( dword pNumSetLODs ); 
    dword __fastcall FGetNumSetPriorities(); 
    void __fastcall FSetNumSetPriorities( dword pNumSetPriorities ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DDEVINFO_TEXTURING* __fastcall FGetInternal_D3DDEVINFO_TEXTURING_Ptr(); 
    void __fastcall FSetInternal_D3DDEVINFO_TEXTURING( D3DDEVINFO_TEXTURING* 
    pD3DDEVINFO_TEXTURING ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DDEVINFO_TEXTURING fD3DDEVINFO_TEXTURING; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
