#ifndef TD3DMaterialH
#define TD3DMaterialH

// ==========================================================================
// File:         TD3DMaterial.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TD3DMaterial Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TD3DMaterial properties.
#include "TD3DColorValue.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DMaterial
// Description: The TD3DMaterial component wraps the DirectX D3DMATERIAL7 
//              structure, which specifies the color properties of a material 
//              in response to lighting. 
//              
//              This component is used by the TDx_3DDevice::GetMaterial() and 
//              TDx_3DDevice::SetMaterial() methods when retrieving or 
//              modifying material properties. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DMaterial : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DMaterial : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Ambient
    // Description: The Ambient property contains a TD3DColorValue component 
    //              defining how the material reflects ambient lighting, thus 
    //              helping determine the apparent color of the material. 
    //              
    //              This property is unioned with the ambient property of 
    //              this component. 
    //              Ambient values usually have less impact than Diffuse, due 
    //              to most scenes having more diffuse lighting than ambient. 
    // ----------------------------------------------------------------------
    __property TD3DColorValue* Ambient = { read=FGetAmbient, 
    write=FSetAmbient, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    Diffuse
    // Description: The Diffuse property contains a TD3DColorValue component 
    //              defining how the material reflects diffuse lighting, thus 
    //              helping determine the apparent color of the material. 
    //              
    //              This property is unioned with the diffuse property of 
    //              this component. 
    //              Diffuse values usually have greater impact than Ambient, 
    //              due to most scenes having more diffuse lighting than 
    //              ambient. 
    // ----------------------------------------------------------------------
    __property TD3DColorValue* Diffuse = { read=FGetDiffuse, 
    write=FSetDiffuse, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    Emissive
    // Description: The Emissive property contains a TD3DColorValue component 
    //              defining how the material emits, possibly making the 
    //              material appear to glow independant of light sources. 
    //              
    //              This property is unioned with the emissive property of 
    //              this component. 
    //              Emitted light is not a computational light source and 
    //              will not be reflected by other objects, only affecting 
    //              the material itself. 
    // ----------------------------------------------------------------------
    __property TD3DColorValue* Emissive = { read=FGetEmissive, 
    write=FSetEmissive, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    Specular
    // Description: The Specular property contains a TD3DColorValue component 
    //              defining the color of the specular highlights the 
    //              material reflects under lighting. 
    //              
    //              This property is unioned with the specular property of 
    //              this component. 
    //              Power defines the sharpness and Specular defines the 
    //              color of the highlight. 
    // ----------------------------------------------------------------------
    __property TD3DColorValue* Specular = { read=FGetSpecular, 
    write=FSetSpecular, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    ambient
    // Description: The ambient property contains a TD3DColorValue component 
    //              defining how the material reflects ambient lighting, thus 
    //              helping determine the apparent color of the material. 
    //              
    //              This property is unioned with the Ambient property of 
    //              this compponent. 
    //              Ambient values usually have less impact than Diffuse, due 
    //              to most scenes having more diffuse lighting than ambient. 
    // ----------------------------------------------------------------------
    __property TD3DColorValue* ambient = { read=FGetambient, 
    write=FSetambient, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    diffuse
    // Description: The diffuse property contains a TD3DColorValue component 
    //              defining how the material reflects diffuse lighting, thus 
    //              helping determine the apparent color of the material. 
    //              
    //              This property is unioned with the Diffuse property of 
    //              this component. 
    //              Diffuse values usually have greater impact than Ambient, 
    //              due to most scenes having more diffuse lighting than 
    //              ambient. 
    // ----------------------------------------------------------------------
    __property TD3DColorValue* diffuse = { read=FGetdiffuse, 
    write=FSetdiffuse, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    emissive
    // Description: The emissive property contains a TD3DColorValue component 
    //              defining how the material emits, possibly making the 
    //              material appear to glow independant of light sources. 
    //              
    //              This property is unioned with the Emissive property of 
    //              this component. 
    //              Emitted light is not a computational light source and 
    //              will not be reflected by other objects, only affecting 
    //              the material itself. 
    // ----------------------------------------------------------------------
    __property TD3DColorValue* emissive = { read=FGetemissive, 
    write=FSetemissive, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    specular
    // Description: The specular property contains a TD3DColorValue component 
    //              defining the color of the specular highlights the 
    //              material reflects under lighting. 
    //              
    //              This property is unioned with the Specular property of 
    //              this component. 
    //              Power defines the sharpness and Specular defines the 
    //              color of the highlight. 
    // ----------------------------------------------------------------------
    __property TD3DColorValue* specular = { read=FGetspecular, 
    write=FSetspecular, default=NULL }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DMaterial component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DMaterial method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Power
    // Description: The Power property contains a TD3DColorValue component 
    //              defining the sharpness of the specular highlights the 
    //              material reflects, in essence determining how shiny it 
    //              looks under lighting. 
    //              
    //              This property is unioned with the power property of this 
    //              component. 
    //              Power defines the sharpness and Specular defines the 
    //              color of the highlight. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Power = { read=FGetPower, write=FSetPower, nodefault 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal D3DMATERIAL7 structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    power
    // Description: The power property contains a TD3DColorValue component 
    //              defining the sharpness of the specular highlights the 
    //              material reflects, in essence determining how shiny it 
    //              looks under lighting. 
    //              
    //              This property is unioned with the Power property of this 
    //              component. 
    //              Power defines the sharpness and Specular defines the 
    //              color of the highlight. 
    // ----------------------------------------------------------------------
    __property D3DVALUE power = { read=FGetpower, write=FSetpower, nodefault 
    }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DMaterial's internal D3DMATERIAL7 structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DMaterial(TComponent* Owner); 
    virtual __fastcall ~TD3DMaterial(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DMATERIAL7* Internal_D3DMATERIAL7_Ptr = { 
    read=FGetInternal_D3DMATERIAL7_Ptr, nodefault }; 

    void __fastcall Internal_D3DMATERIAL7_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    TD3DColorValue* __fastcall FGetAmbient(); 
    void __fastcall FSetAmbient( TD3DColorValue* pAmbient ); 
    TD3DColorValue* __fastcall FGetDiffuse(); 
    void __fastcall FSetDiffuse( TD3DColorValue* pDiffuse ); 
    TD3DColorValue* __fastcall FGetEmissive(); 
    void __fastcall FSetEmissive( TD3DColorValue* pEmissive ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    D3DVALUE __fastcall FGetPower(); 
    void __fastcall FSetPower( D3DVALUE pPower ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    TD3DColorValue* __fastcall FGetSpecular(); 
    void __fastcall FSetSpecular( TD3DColorValue* pSpecular ); 
    TD3DColorValue* __fastcall FGetambient(); 
    void __fastcall FSetambient( TD3DColorValue* pambient ); 
    TD3DColorValue* __fastcall FGetdiffuse(); 
    void __fastcall FSetdiffuse( TD3DColorValue* pdiffuse ); 
    TD3DColorValue* __fastcall FGetemissive(); 
    void __fastcall FSetemissive( TD3DColorValue* pemissive ); 
    D3DVALUE __fastcall FGetpower(); 
    void __fastcall FSetpower( D3DVALUE ppower ); 
    TD3DColorValue* __fastcall FGetspecular(); 
    void __fastcall FSetspecular( TD3DColorValue* pspecular ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DMATERIAL7* __fastcall FGetInternal_D3DMATERIAL7_Ptr(); 
    void __fastcall FSetInternal_D3DMATERIAL7( D3DMATERIAL7* pD3DMATERIAL7 ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    TD3DColorValue* fAmbient; 
    TD3DColorValue* fDiffuse; 
    TD3DColorValue* fEmissive; 
    HRESULT fErrorValue; 
    dword fSize; 
    TD3DColorValue* fSpecular; 
    TD3DColorValue* fambient; 
    TD3DColorValue* fdiffuse; 
    TD3DColorValue* femissive; 
    TD3DColorValue* fspecular; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DMATERIAL7 fD3DMATERIAL7; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
