#ifndef TD3DVectorH
#define TD3DVectorH

// ==========================================================================
// File:         TD3DVector.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TD3DVector Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DVector
// Description: The TD3DVector component wraps multiple DirectX D3DVECTOR 
//              structures that define vectors. 
//              
//              This component defines a couple of properties of the 
//              TD3DLight component as weill as being used in the 
//              TDx_3DDevice::ComputeSphereVisibility() method. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DVector : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DVector : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DVector component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              D3DVECTOR structures wrapped inside the TD3DVector 
    //              component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TD3DVector component wraps a single D3DVECTOR 
    //              structure. To wrap multiple structures, set the 
    //              TD3DVector::ArraySize to the total number of internal 
    //              structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DVector method or fget/fset. 
    //              eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of a single 
    //              internal D3DVECTOR structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal D3DVECTOR structures. This result is the same as 
    //              multiplying TD3DVector::ArraySize and TD3DVector::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    X
    // Description: The X property defines the X axis value of the vector. 
    //              
    //              This property is unioned with the x property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE X[dword pArrayIndex] = { read=FGetX, write=FSetX }; 

    // ----------------------------------------------------------------------
    // Property:    Y
    // Description: The Y property defines the Y axis value of the vector. 
    //              
    //              This property is unioned with the y property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Y[dword pArrayIndex] = { read=FGetY, write=FSetY }; 

    // ----------------------------------------------------------------------
    // Property:    Z
    // Description: The Z property defines the Z axis value of the vector. 
    //              
    //              This property is unioned with the z property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Z[dword pArrayIndex] = { read=FGetZ, write=FSetZ }; 

    // ----------------------------------------------------------------------
    // Property:    x
    // Description: The x property defines the X axis value of the vector. 
    //              
    //              This property is unioned with the X property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE x[dword pArrayIndex] = { read=FGetx, write=FSetx }; 

    // ----------------------------------------------------------------------
    // Property:    y
    // Description: The y property defines the Y axis value of the vector. 
    //              
    //              This property is unioned with the Y property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE y[dword pArrayIndex] = { read=FGety, write=FSety }; 

    // ----------------------------------------------------------------------
    // Property:    z
    // Description: The z property defines the Z axis value of the vector. 
    //              
    //              This property is unioned with the Z property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE z[dword pArrayIndex] = { read=FGetz, write=FSetz }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DVector's internal D3DVECTOR structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TD3DVector::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal D3DVECTOR structures of the 
    //              TD3DVector component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DVector(TComponent* Owner); 
    virtual __fastcall ~TD3DVector(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DVECTOR* Internal_D3DVECTOR_Ptr = { 
    read=FGetInternal_D3DVECTOR_Ptr, nodefault }; 

    void __fastcall Internal_D3DVECTOR_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetSizeAll(); 
    D3DVALUE __fastcall FGetX( dword pArrayIndex ); 
    void __fastcall FSetX( dword pArrayIndex, D3DVALUE pX ); 
    D3DVALUE __fastcall FGetY( dword pArrayIndex ); 
    void __fastcall FSetY( dword pArrayIndex, D3DVALUE pY ); 
    D3DVALUE __fastcall FGetZ( dword pArrayIndex ); 
    void __fastcall FSetZ( dword pArrayIndex, D3DVALUE pZ ); 
    D3DVALUE __fastcall FGetx( dword pArrayIndex ); 
    void __fastcall FSetx( dword pArrayIndex, D3DVALUE px ); 
    D3DVALUE __fastcall FGety( dword pArrayIndex ); 
    void __fastcall FSety( dword pArrayIndex, D3DVALUE py ); 
    D3DVALUE __fastcall FGetz( dword pArrayIndex ); 
    void __fastcall FSetz( dword pArrayIndex, D3DVALUE pz ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DVECTOR* __fastcall FGetInternal_D3DVECTOR_Ptr(); 
    void __fastcall FSetInternal_D3DVECTOR( dword pArrayIndex, D3DVECTOR* 
    pD3DVECTOR ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    HRESULT fErrorValue; 
    dword fSize; 
    dword fSizeAll; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DVECTOR* fD3DVECTOR; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
