#ifndef TD3DVertexBufferDescH
#define TD3DVertexBufferDescH

// ==========================================================================
// File:         TD3DVertexBufferDesc.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TD3DVertexBufferDesc Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"

// ==========================================================================
// Set:         TD3DVertexBufferDescCaps_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TD3DVertexBufferDesc::Caps property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Caps property description below. 
// ==========================================================================
enum TD3DVertexBufferDesc_Caps_Flags
    {
    d3dvbcaps_donotclip,
    d3dvbcaps_optimized,
    d3dvbcaps_systemmemory,
    d3dvbcaps_writeonly
    };
typedef Set< TD3DVertexBufferDesc_Caps_Flags, 
             d3dvbcaps_donotclip, 
             d3dvbcaps_writeonly >  TD3DVertexBufferDesc_Caps_Set;
// ==========================================================================
// Set:         TD3DVertexBufferDescFVF_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TD3DVertexBufferDesc::FVF property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              FVF property description below. 
// ==========================================================================
enum TD3DVertexBufferDesc_FVF_Flags
    {
    d3dfvf_diffuse,
    d3dfvf_normal,
    d3dfvf_position_mask,
    d3dfvf_reserved0,
    d3dfvf_reserved1,
    d3dfvf_reserved2,
    d3dfvf_specular,
    d3dfvf_tex0,
    d3dfvf_tex1,
    d3dfvf_tex2,
    d3dfvf_tex3,
    d3dfvf_tex4,
    d3dfvf_tex5,
    d3dfvf_tex6,
    d3dfvf_tex7,
    d3dfvf_tex8,
    d3dfvf_texcount_mask,
    d3dfvf_texcount_shift,
    d3dfvf_textureformat1,
    d3dfvf_textureformat2,
    d3dfvf_textureformat3,
    d3dfvf_textureformat4,
    d3dfvf_xyz,
    d3dfvf_xyzb1,
    d3dfvf_xyzb2,
    d3dfvf_xyzb3,
    d3dfvf_xyzb4,
    d3dfvf_xyzb5,
    d3dfvf_xyzrhw
    };
typedef Set< TD3DVertexBufferDesc_FVF_Flags, 
             d3dfvf_diffuse, 
             d3dfvf_xyzrhw >  TD3DVertexBufferDesc_FVF_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DVertexBufferDesc
// Description: The TD3DVertexBufferDesc component wraps the DirectX 
//              D3DVertexBufferDesc structure which describes a vertex 
//              buffer.   
//              
//              This component is used by the TDx_3D::CreateVertexBuffer() 
//              and TDx_3DVertexBuffer::GetVertexBufferDesc() methods when 
//              creating or retrieving the properties of a vertex buffer. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DVertexBufferDesc : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DVertexBufferDesc : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Caps_
    // Description: The Caps property describes the vertex buffer 
    //              capabilities and settings. 
    //              The described effect applies when the flag is set. 
    //              
    //              Setting no flags creates the vertex buffer in memory 
    //              optimal for efficient read operations. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Caps' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Caps 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TD3DVertexBufferDesc_Caps_Set Caps_ = { read=FGetCaps_, 
    write=FSetCaps_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    FVF_
    // Description: The FVF property uses flexible vertex format flags to 
    //              describe the verticies in this buffer. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'FVF' property via the BCB Object Inspector.  For 
    //              detailed descriptions of each flag, see the FVF property 
    //              description below. 
    // ----------------------------------------------------------------------
    __property TD3DVertexBufferDesc_FVF_Set FVF_ = { read=FGetFVF_, 
    write=FSetFVF_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    NumVertices
    // Description: The NumVertices property defines how many verticies the 
    //              buffer can hold. 
    //              
    //              The maximum value is defined by D3DMAXNUMVERTICES and is 
    //              usually (0xFFFF). 
    // ----------------------------------------------------------------------
    __property dword NumVertices = { read=FGetNumVertices, 
    write=FSetNumVertices, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The size of the internal structure, initialized in the 
    //              constructor. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(D3DVERTEXBUFFERDESC) }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DVertexBufferDesc component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Caps
    // Description: The Caps property describes the vertex buffer 
    //              capabilities and settings. 
    //              The described effect applies when the flag is set. 
    //              
    //              Setting no flags creates the vertex buffer in memory 
    //              optimal for efficient read operations. 
    // Flags:       D3DVBCAPS_DONOTCLIP -
    //                No clipping information may be stored in the vertex 
    //                buffer. 
    //              D3DVBCAPS_OPTIMIZED -
    //                Optimized vertex data may be stored in the vertex 
    //                buffer. 
    //                
    //                This flag is not used when creating a new vertex 
    //                buffer. 
    //              D3DVBCAPS_SYSTEMMEMORY -
    //                Create the vertex buffer in system memory. 
    //                
    //                Optimal for software rendering. 
    //              D3DVBCAPS_WRITEONLY -
    //                Create the vertex buffer in memory optimal for 
    //                efficient write and rendering operations. 
    //                
    //                Read attempts on buffers created with this setting may 
    //                result in degraded performance. 
    // ----------------------------------------------------------------------
    __property dword Caps = { read=FGetCaps, write=FSetCaps, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DVertexBufferDesc method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    FVF
    // Description: The FVF property uses flexible vertex format flags to 
    //              describe the verticies in this buffer. 
    //              The described effect applies when the flag is set. 
    // Flags:       D3DFVF_DIFFUSE -
    //                Vertices have a diffuse color component. 
    //              D3DFVF_NORMAL -
    //                Vertices have a normal vector to define orientation. 
    //                
    //                Mutually exclusive with D3DFVF_XYZRHW. 
    //              D3DFVF_POSITION_MASK -
    //                Mask for position bits. 
    //              D3DFVF_RESERVED0 -
    //                Mask values for reserved bits in the flexible vertex 
    //                format. 
    //                Do not use. 
    //              D3DFVF_RESERVED1 -
    //                Emulate TD3DLVertex processing. 
    //                
    //                D3DFVF_XYZ, D3DFVF_DIFFUSE, D3DFVF_SPECULAR and 
    //                D3DFVF_TEX1 must also be set. 
    //              D3DFVF_RESERVED2 -
    //                Mask values for reserved bits in the flexible vertex 
    //                format. 
    //                Do not use. 
    //              D3DFVF_SPECULAR -
    //                Vertices have a specular color component. 
    //              D3DFVF_TEX0 -
    //                Vertices have no texture coordinate set. 
    //              D3DFVF_TEX1 -
    //                Vertices have a texture coordinate set. 
    //              D3DFVF_TEX2 -
    //                Vertices have 2 texture coordinate sets. 
    //              D3DFVF_TEX3 -
    //                Vertices have 3 texture coordinate sets. 
    //              D3DFVF_TEX4 -
    //                Vertices have 4 texture coordinate sets. 
    //              D3DFVF_TEX5 -
    //                Vertices have 5 texture coordinate sets. 
    //              D3DFVF_TEX6 -
    //                Vertices have 6 texture coordinate sets. 
    //              D3DFVF_TEX7 -
    //                Vertices have 7 texture coordinate sets. 
    //              D3DFVF_TEX8 -
    //                Vertices have 8 texture coordinate sets. 
    //              D3DFVF_TEXCOUNT_MASK -
    //                Mask value for texture flag bits. 
    //              D3DFVF_TEXCOUNT_SHIFT -
    //                The number of bits by which to shift an integer value 
    //                that identifies the number of a texture coordinates for 
    //                a vertex. 
    //              D3DFVF_TEXTUREFORMAT1 -
    //                Vertices have one dimensional texture coordinates. 
    //              D3DFVF_TEXTUREFORMAT2 -
    //                Vertices have two dimensional texture coordinates. 
    //              D3DFVF_TEXTUREFORMAT3 -
    //                Vertices have three dimensional texture coordinates. 
    //              D3DFVF_TEXTUREFORMAT4 -
    //                Vertices have four dimensional texture coordinates. 
    //              D3DFVF_XYZ -
    //                Untransformed vertices have their position defined in 
    //                terms of (X,Y,Z). 
    //                
    //                Mutually exclusive with D3DFVF_XYZRHW. 
    //                D3DFVF_NORMAL is required and either a color component 
    //                (D3DFVF_DIFFUSE or D3DFVF_SPECULAR) or at least one 
    //                texture coordinate (D3DFVF_TEX1 through D3DFVF_TEX8) 
    //                must be specified. 
    //              D3DFVF_XYZB1 -
    //                Vertices contain position data along with a 
    //                corresponding weighting (beta) value for multimatrix 
    //                vertex blending. 
    //                
    //                Blending with up to 3 weighting values and 4 blending 
    //                matrices is currently supported. 
    //              D3DFVF_XYZB2 -
    //                Vertices contain position data along with 2 
    //                corresponding weighting (beta) values for multimatrix 
    //                vertex blending. 
    //                
    //                Blending with up to 3 weighting values and 4 blending 
    //                matrices is currently supported. 
    //              D3DFVF_XYZB3 -
    //                Vertices contain position data along with 3 
    //                corresponding weighting (beta) values for multimatrix 
    //                vertex blending. 
    //                
    //                Blending with up to 3 weighting values and 4 blending 
    //                matrices is currently supported. 
    //              D3DFVF_XYZB4 -
    //                Vertices contain position data along with 4 
    //                corresponding weighting (beta) values for multimatrix 
    //                vertex blending. 
    //                
    //                Blending with up to 3 weighting values and 4 blending 
    //                matrices is currently supported. 
    //              D3DFVF_XYZB5 -
    //                Vertices contain position data along with 5 
    //                corresponding weighting (beta) values for multimatrix 
    //                vertex blending. 
    //                
    //                Blending with up to 3 weighting values and 4 blending 
    //                matrices is currently supported. 
    //              D3DFVF_XYZRHW -
    //                Transformed vertices have their position defined in 
    //                terms of (X,Y,Z). 
    //                
    //                Mutually exclusive with D3DFVF_NORMAL and D3DFVF_XYZ. 
    //                A color component (D3DFVF_DIFFUSE or D3DFVF_SPECULAR) 
    //                or at least one texture coordinate (D3DFVF_TEX1 through 
    //                D3DFVF_TEX8) must be specified. 
    // ----------------------------------------------------------------------
    __property dword FVF = { read=FGetFVF, write=FSetFVF, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DVertexBufferDesc's internal D3DVERTEXBUFFERDESC 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Caps_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Caps property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Caps_Strings = { read=FGetCaps_Strings };

    // ----------------------------------------------------------------------
    // Property:    FVF_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the FVF property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* FVF_Strings = { read=FGetFVF_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DVertexBufferDesc(TComponent* Owner); 
    virtual __fastcall ~TD3DVertexBufferDesc(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DVERTEXBUFFERDESC* Internal_D3DVERTEXBUFFERDESC_Ptr = { 
    read=FGetInternal_D3DVERTEXBUFFERDESC_Ptr, nodefault }; 

    void __fastcall Internal_D3DVERTEXBUFFERDESC_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TD3DVertexBufferDesc_Caps_Set __fastcall FGetCaps_(); 
    void __fastcall FSetCaps_( TD3DVertexBufferDesc_Caps_Set pCaps_ ); 
    TD3DVertexBufferDesc_FVF_Set __fastcall FGetFVF_(); 
    void __fastcall FSetFVF_( TD3DVertexBufferDesc_FVF_Set pFVF_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetCaps(); 
    void __fastcall FSetCaps( dword pCaps ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFVF(); 
    void __fastcall FSetFVF( dword pFVF ); 
    dword __fastcall FGetNumVertices(); 
    void __fastcall FSetNumVertices( dword pNumVertices ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetCaps_Strings();

    TStringList* __fastcall FGetFVF_Strings();
// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DVERTEXBUFFERDESC* __fastcall FGetInternal_D3DVERTEXBUFFERDESC_Ptr(); 
    void __fastcall FSetInternal_D3DVERTEXBUFFERDESC( D3DVERTEXBUFFERDESC* 
    pD3DVERTEXBUFFERDESC ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fCaps_Strings;
    TStringList* fFVF_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DVERTEXBUFFERDESC fD3DVERTEXBUFFERDESC; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
