#ifndef TD3DViewPortH
#define TD3DViewPortH

// ==========================================================================
// File:         TD3DViewPort.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TD3DViewPort Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DViewPort
// Description: The TD3DViewPort component wraps the DirectX D3DVIEWPORT7 
//              structure which defines the surface rectangle into which a 
//              three-dimensional scene is projected.   
//              
//              This component is used by the TDx_3DDevice::GetViewport() and 
//              TDx_3DDevice::SetViewPort() methods. 
//              
//              A new transformation matrix is built when the viewport is 
//              changed with SetViewPort. 
//              Viewport dimensions are given relative to the top left of the 
//              device with the y value increasing as you go down the screen. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DViewPort : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DViewPort : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Height
    // Description: The Height property defines the height of the viewport, 
    //              in pixels. 
    // ----------------------------------------------------------------------
    __property dword Height = { read=FGetHeight, write=FSetHeight, default=0 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Width
    // Description: The Width property defines the width of the viewport, in 
    //              pixels. 
    // ----------------------------------------------------------------------
    __property dword Width = { read=FGetWidth, write=FSetWidth, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    X
    // Description: The X property defines the X coordinate of the upper left 
    //              corner of the viewport. 
    //              
    //              Unless rendering to a subarea of the surface, this is 
    //              usually 0. 
    // ----------------------------------------------------------------------
    __property dword X = { read=FGetX, write=FSetX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Y
    // Description: The Y property defines the Y coordinate of the upper left 
    //              corner of the viewport. 
    //              
    //              Unless rendering to a subarea of the surface, this is 
    //              usually 0. 
    // ----------------------------------------------------------------------
    __property dword Y = { read=FGetY, write=FSetY, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DViewPort component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DViewPort method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    MaxZ
    // Description: The MaxZ property contains a D3DVALUE defining the 
    //              maximum nonhomogenous Z coordinate. 
    //              
    //              Usually this value is 1.0 
    // ----------------------------------------------------------------------
    __property D3DVALUE MaxZ = { read=FGetMaxZ, write=FSetMaxZ, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    MinZ
    // Description: The MinZ property contains a D3DVALUE defining the 
    //              minimum nonhomogenous Z coordinate. 
    //              
    //              Usually this value is 0.0 
    // ----------------------------------------------------------------------
    __property D3DVALUE MinZ = { read=FGetMinZ, write=FSetMinZ, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal D3DVIEWPORT7 structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DViewPort's internal D3DVIEWPORT7 structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DViewPort(TComponent* Owner); 
    virtual __fastcall ~TD3DViewPort(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DVIEWPORT7* Internal_D3DVIEWPORT7_Ptr = { 
    read=FGetInternal_D3DVIEWPORT7_Ptr, nodefault }; 

    void __fastcall Internal_D3DVIEWPORT7_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetHeight(); 
    void __fastcall FSetHeight( dword pHeight ); 
    D3DVALUE __fastcall FGetMaxZ(); 
    void __fastcall FSetMaxZ( D3DVALUE pMaxZ ); 
    D3DVALUE __fastcall FGetMinZ(); 
    void __fastcall FSetMinZ( D3DVALUE pMinZ ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetWidth(); 
    void __fastcall FSetWidth( dword pWidth ); 
    dword __fastcall FGetX(); 
    void __fastcall FSetX( dword pX ); 
    dword __fastcall FGetY(); 
    void __fastcall FSetY( dword pY ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DVIEWPORT7* __fastcall FGetInternal_D3DVIEWPORT7_Ptr(); 
    void __fastcall FSetInternal_D3DVIEWPORT7( D3DVIEWPORT7* pD3DVIEWPORT7 ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DVIEWPORT7 fD3DVIEWPORT7; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
