// --------------------------------------------------------------------------

// ==========================================================================
// File:         TDx_3D.CPP
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file contains the code for the TDx_3D Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes ...
// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
// --------------------------------------------------------------------------
#include "TDx_3D.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_3D component.
#include "TD3DDeviceDesc.h"
#include "TDDPixelFormat.h"
#include "TDx_Draw.h"
// --------------------------------------------------------------------------
// external classes used by TDx_3D methods.
#include "TDx_DrawSurface.H"
#include "TDx_3DDevice.H"
#include "TD3DVertexBufferDesc.H"
#include "TDx_3DVertexBuffer.H"
// --------------------------------------------------------------------------
#pragma link "TDx_Library_Defns"
#pragma link "TDx_Library_Functions"
#pragma link "TDx_3DI_Library_Defns"
#pragma link "TDx_3DI_Library_Functions"

// --------------------------------------------------------------------------
// Object Registration...
// --------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    #pragma package(smart_init)
#endif
// --------------------------------------------------------------------------
static inline void ValidCtrCheck(TDx_3D*)
{
    new TDx_3D(NULL);
}
// --------------------------------------------------------------------------
namespace Tdx_3d
{
    #if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    void __fastcall PACKAGE Register()
    #else
    void __fastcall Register()
    #endif
    {
        TComponentClass classes[1] = {__classid(TDx_3D)};
        RegisterComponents("TDx_3DI", classes, 0);
    }
}

// --------------------------------------------------------------------------
// Extra code for Callback Handling
// --------------------------------------------------------------------------
TDx_3D* TDx_3D_OnEnumDevicesOwner = NULL;
TDx_3D* TDx_3D_OnEnumZBufferFormatsOwner = NULL;

// --------------------------------------------------------------------------
// Constructor: TDx_3D::TDx_3D()
// Description: The default constructor for the TDx_3D object.
// --------------------------------------------------------------------------
__fastcall TDx_3D::TDx_3D(TComponent* Owner)
    : TComponent(Owner)
{
fCreated = false;
fErrorValue = D3D_OK;
}
// --------------------------------------------------------------------------
// Destructor:  TDx_3D::~TDx_3D()
// Description: The destructor for the TDx_3D object.
// --------------------------------------------------------------------------
__fastcall TDx_3D::~TDx_3D()
{
// destroy internals
if (Created) Destroy();
}
// --------------------------------------------------------------------------
// Property:    Created
// Description: The Created property is true if the internal LPDIRECT3D7 used 
//              in this component has been successfully created, otherwise 
//              Created is false. 
//              
//              To create the internal LPDIRECT3D7, call the TDx_3D::Create() 
//              method. 
// --------------------------------------------------------------------------
bool __fastcall TDx_3D::FGetCreated()
{
return fCreated;
}
// --------------------------------------------------------------------------
void __fastcall TDx_3D::FSetCreated( bool pCreated )
{
fCreated = (pCreated && (fLPDIRECT3D7==NULL));
}
// --------------------------------------------------------------------------
// Property:    ErrorValue
// Description: The ErrorValue property contains the last error value 
//              returned from a call to a TDx_3D method or fget/fset. eg. 
//              D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
// --------------------------------------------------------------------------
HRESULT __fastcall TDx_3D::FGetErrorValue()
{
return fErrorValue;
}
// --------------------------------------------------------------------------
void __fastcall TDx_3D::FSetErrorValue( HRESULT pErrorValue )
{
fErrorValue = pErrorValue;
}
// --------------------------------------------------------------------------
// Method:      Create()
// Description: The Create() method is used to automatically create the 
//              internal LPDIRECT3D7 interface used in the TDx_3D component 
//              and must be called before any methods of the TDx_3D component 
//              will function. 
// Params:      pDx_Draw - 
//                The Dx_Draw parameter ... 
// --------------------------------------------------------------------------
bool __fastcall TDx_3D::Create( TDx_Draw* pDx_Draw )
{
// if the component internals have already been created, exit
if (fCreated)
    {
    fErrorValue = TDX_ALREADYCREATED;
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_ALREADYCREATED", "The "+Name+"::Create() method was called when the component has already been created." );
    return false;
    }

// check parameters
if (pDx_Draw==NULL)
	{
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_ERROR", Name+"::Create() requires you supply a DirectDraw device created using TDx_Draw::Create()" );
    return false;
    }

// create the internal interface
fErrorValue = pDx_Draw->Internal_LPDIRECTDRAW7->QueryInterface( IID_IDirect3D7, (VOID**) &fLPDIRECT3D7 );

// check for error result
if (fErrorValue!=D3D_OK)
    {
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_ERROR", Name+"::Create() failed with Error: "+TDx_3DI_Library_ErrorString(fErrorValue)+", "+TDx_3DI_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// successfully created
fCreated = true;
if (FOnCreate) FOnCreate(this);
return true;
}

// --------------------------------------------------------------------------
// Method:      CreateDevice()
// Description: The TDx_3D::CreateDevice() method will create a TDx_3DDevice 
//              interface which can then be used to access the draw primitive 
//              methods. 
//              
//              Multiple rendering devices can be created from a single 
//              TDx_3D component. However, because they share the same 
//              physical resources, performance will be significantly 
//              affected. 
// Params:      pClsId - 
//                The ClsId parameter identifies the type of device being 
//                created. 
//                
//                The possible values are: 
//                
//                IID_IDirect3DTnLHalDevice 
//                IID_IDirect3DHALDevice 
//                IID_IDirect3DMMXDevice 
//                IID_IDirect3DRGBDevice 
//              pDDS - 
//                The DDS parameter references the new 3D device's render 
//                target surface. 
//                
//                The surface must have been created with the DDSCAPS_3DDEVICE 
//                flag set. 
//              pD3DDevice - 
//                The D3DDevice property will reference the address of the new 
//                3D device if the method returns successfully. 
// --------------------------------------------------------------------------
bool __fastcall TDx_3D::CreateDevice( REFCLSID pClsId, TDx_DrawSurface* pDDS, TDx_3DDevice* pD3DDevice )
{
// Original Function Definition
// HRESULT CreateDevice( 
//   REFCLSID rclsid,                     
//   LPDIRECTDRAWSURFACE7 lpDDS, 
//   LPDIRECT3DDEVICE7* lplpD3DDevice 
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::CreateDevice()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECT3D7->CreateDevice( pClsId, (pDDS==NULL) ? NULL : pDDS->Internal_LPDIRECTDRAWSURFACE7, (pD3DDevice==NULL) ? NULL : pD3DDevice->Internal_LPDIRECT3DDEVICE7_Ptr );

// Translate Data returned from Function
if (pD3DDevice!=NULL) pD3DDevice->Internal_LPDIRECT3DDEVICE7_Update();

// Handle any Known Results
if (fErrorValue!=D3D_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::CreateDevice()", TDx_3DI_Library_ErrorString(fErrorValue), TDx_3DI_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      CreateVertexBuffer()
// Description: The TDx_3D::CreateVertexBuffer() method will create a 
//              TDx_VertexBuffer interface which can then be used to 
//              manipulate vertex buffered vertices. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              CLASS_E_NOAGGREGATION   
//              D3DERR_INVALIDVERTEXFORMAT 
//              D3DERR_VBUF_CREATE_FAILED 
//              DDERR_INVALIDOBJECT   
//              DDERR_INVALIDPARAMS 
//              DDERR_OUTOFMEMORY 
// Params:      pVBDesc - 
//                The VBDesc parameter references the TD3DVertexBufferDesc 
//                component describing the number and format of the vertices 
//                the new vertex buffer will accomodate. 
//              pD3DVertexBuffer - 
//                The D3DVertexBuffer paremeter references the new vertex 
//                buffer if the method returns successfully. 
// --------------------------------------------------------------------------
bool __fastcall TDx_3D::CreateVertexBuffer( TD3DVertexBufferDesc* pVBDesc, TDx_3DVertexBuffer* pD3DVertexBuffer )
{
// Original Function Definition
// HRESULT CreateVertexBuffer( 
//   LPD3DVERTEXBUFFERDESC    lpVBDesc, 
//   LPDIRECT3DVERTEXBUFFER7 *lplpD3DVertexBuffer, 
//    DWORD dwFlags 
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::CreateVertexBuffer()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECT3D7->CreateVertexBuffer( (pVBDesc==NULL) ? NULL : pVBDesc->Internal_D3DVERTEXBUFFERDESC_Ptr, (pD3DVertexBuffer==NULL) ? NULL : pD3DVertexBuffer->Internal_LPDIRECT3DVERTEXBUFFER7_Ptr, 0 );

// Translate Data returned from Function
if (pVBDesc!=NULL) pVBDesc->Internal_D3DVERTEXBUFFERDESC_Update();
if (pD3DVertexBuffer!=NULL) pD3DVertexBuffer->Internal_LPDIRECT3DVERTEXBUFFER7_Update();

// Handle any Known Results
if (fErrorValue!=D3D_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::CreateVertexBuffer()", TDx_3DI_Library_ErrorString(fErrorValue), TDx_3DI_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      Destroy()
// Description: The Destroy() method is used to automatically destroy the 
//              internal LPDIRECT3D7 interface used in the TDx_3D component 
//              and should be called when the internal interface is no longer 
//              required. 
//              
//              Note: This method is called by the component's destructor. 
// --------------------------------------------------------------------------
bool __fastcall TDx_3D::Destroy()
{
// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_NOTCREATED", "The "+Name+"::Destroy() method was called before being created successfully." );
    return false;
    }

// code that destroys the internal interfaces
if (FOnDestroy) FOnDestroy(this);

if (fLPDIRECT3D7==NULL)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_ERROR", "The "+Name+" component's internal fLPDIRECT3D7 is NULL. Aborting the Destroy()." );
    return false;
    }

try {
    fLPDIRECT3D7->Release();
    }
catch (...)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_ERROR", "An exception occurred in fLPDIRECT3D7->Release(). Check you are destroying all components in reverse order of creation." );
    }

// successful destruction
fCreated = false;
return true;
}

// --------------------------------------------------------------------------
// Method:      EnumDevices()
// Description: The TDx3D::EnumDevices() method enumerates the currently 
//              installed TDx_3D compatible device drivers. 
//              
//              This method will trigger an TDx_3D::OnEnumDevices() event for 
//              each device enumerated. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT   
//              DDERR_INVALIDPARAMS 
// Params:      pUserArg - 
//                The UserArg parameter references application defined data to 
//                be passed to the TDx_3D::EnumDevicesCallback() event each 
//                time it is called. 
// --------------------------------------------------------------------------
bool __fastcall TDx_3D::EnumDevices( void* pUserArg )
{
// Original Function Definition
// HRESULT EnumDevices( 
//   LPD3DENUMDEVICESCALLBACK lpEnumDevicesCallback,   
//   LPVOID lpUserArg                                 
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::EnumDevices()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
TDx_3D_OnEnumDevicesOwner = this;
fErrorValue = fLPDIRECT3D7->EnumDevices( Internal_EnumDevicesCallback, (LPVOID) pUserArg );
TDx_3D_OnEnumDevicesOwner = NULL;

// Handle any Known Results
if (fErrorValue!=D3D_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::EnumDevices()", TDx_3DI_Library_ErrorString(fErrorValue), TDx_3DI_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      EnumZBufferFormats()
// Description: The TDx_3D::EnumZBufferFormats() method will enumerate the 
//              supported z-buffer formats for a specified device. 
//              
//              This method will trigger a TDx_3D::OnEnumZBufferFormats() 
//              event for each z-buffer format enumerated. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT   
//              DDERR_INVALIDPARAMS   
//              DDERR_NOZBUFFERHW 
//              DDERR_OUTOFMEMORY 
// Params:      pDevice - 
//                The Device parameter identifies the device for which 
//                enumeration of z-buffer formats is to be performed. 
//              pContext - 
//                The Context parameter references application defined data to 
//                be passed to the TDx_3D::EnumZBufferFormatsCallback event 
//                each time it is called. 
// --------------------------------------------------------------------------
bool __fastcall TDx_3D::EnumZBufferFormats( REFCLSID pDevice, void* pContext )
{
// Original Function Definition
// HRESULT EnumZBufferFormats( 
//   REFCLSID riidDevice, 
//   LPD3DENUMPIXELFORMATSCALLBACK lpEnumCallback, 
//   LPVOID   lpContext 
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::EnumZBufferFormats()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
TDx_3D_OnEnumZBufferFormatsOwner = this;
fErrorValue = fLPDIRECT3D7->EnumZBufferFormats( pDevice, Internal_EnumZBufferFormatsCallback, (LPVOID) pContext );
TDx_3D_OnEnumZBufferFormatsOwner = NULL;

// Handle any Known Results
if (fErrorValue!=D3D_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::EnumZBufferFormats()", TDx_3DI_Library_ErrorString(fErrorValue), TDx_3DI_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      EvictManagedTextures()
// Description: The TDx_3D::EvictManagedTextures() method removes all managed 
//              textures from memory. 
//              
//              Managed textures are those created with the 
//              DDSCAPS2_TEXTUREMANAGE or DDSCAPS2_D3DTEXTUREMANAGE flags. 
// --------------------------------------------------------------------------
bool __fastcall TDx_3D::EvictManagedTextures()
{
// Original Function Definition
// HRESULT EvictManagedTextures(); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::EvictManagedTextures()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECT3D7->EvictManagedTextures(  );

// Handle any Known Results
if (fErrorValue!=D3D_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::EvictManagedTextures()", TDx_3DI_Library_ErrorString(fErrorValue), TDx_3DI_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      Internal_EnumDevicesCallback()
// Description: The TDx_3D::Internal_EnumDevicesCallback() method is used 
//              internally by the TDx_3DI_Library to redirect a standard 
//              callback function to a BCB style event. 
//              
//              It receives all enumeration callbacks generated by the 
//              TDx_3D::EnumDevices() method and on each enumeration calls 
//              the TDx_3D::OnEnumDevices() event, translating parameters 
//              between DirectX and BCB as required. 
// Params:      pDeviceDescription - 
//                The DeviceDescription parameter references a text description 
//                of the device. 
//              pDeviceName - 
//                The DeviceName parameter references the device name. 
//              pD3DDeviceDesc - 
//                The D3DDeviceDesc parameter references a TD3DDeviceDesc 
//                component containing the device capabilities. 
//              pContext - 
//                The Context parameter references application defined data to 
//                be passed to the callback event. 
// --------------------------------------------------------------------------
HRESULT __stdcall TDx_3D::Internal_EnumDevicesCallback( LPSTR pDeviceDescription, LPSTR pDeviceName, LPD3DDEVICEDESC7 pD3DDeviceDesc, LPVOID pContext )
{
// Original Function Definition
//HRESULT CALLBACK D3DEnumDevicesCallback7(
//  LPSTR lpDeviceDescription,
//  LPSTR lpDeviceName,
//  LPD3DDEVICEDESC7 lpD3DDeviceDesc,
//  LPVOID lpContext
//  };

// Allocate temporary storage for converted data
TD3DDeviceDesc* device_desc = new TD3DDeviceDesc( TDx_3D_OnEnumDevicesOwner );
if (device_desc==NULL)
    {
    if (TDx_3D_OnEnumDevicesOwner->FOnError) TDx_3D_OnEnumDevicesOwner->FOnError( TDx_3D_OnEnumDevicesOwner, TDx_3D_OnEnumDevicesOwner->Name+"::Internal_EnumDevicesCallback()", "TDX_ERROR", "Could not create a temporary TD3DDeviceDesc component." );
    return D3DENUMRET_CANCEL;
    }

// Translate Parameters before calling Original Function
AnsiString DeviceDescription = pDeviceDescription;
AnsiString DeviceName = pDeviceName;
CopyMemory( device_desc->Internal_D3DDEVICEDESC7_Ptr, pD3DDeviceDesc, sizeof(D3DDEVICEDESC7) );
void* Context = pContext;

// transfer to the Callback Event (if it exists)
bool finished = false;

if (TDx_3D_OnEnumDevicesOwner->OnEnumDevices)
    TDx_3D_OnEnumDevicesOwner->OnEnumDevices(
        TDx_3D_OnEnumDevicesOwner, DeviceDescription, DeviceName, device_desc, Context, finished );

// clean up memory
delete device_desc;

// Finished, return FALSE to abort
if (finished) return FALSE;

return TRUE;
}

// --------------------------------------------------------------------------
// Method:      Internal_EnumZBufferFormatsCallback()
// Description: The TDx_3D::Internal_EnumZBufferFormatsCallback() method is 
//              used internally by the TDx_3DI_Library to redirect a standard 
//              callback function to a BCB style event. 
//              
//              It receives all enumeration callbacks generated by the 
//              TDx_3D::EnumZBufferFormats() method and on each enumeration 
//              calls the TDx_3D::OnEnumZBufferFormats() event, translating 
//              parameters between DirectX and BCB as required. 
// Params:      pDDPixFmt - 
//                The DDPixFmt parameter references a TDDPixelFormat component 
//                containing the pixel format of the enumerated z-buffer 
//                format. 
//              pContext - 
//                The Context parameter references application defined data to 
//                be passed to the callback event. 
// --------------------------------------------------------------------------
HRESULT __stdcall TDx_3D::Internal_EnumZBufferFormatsCallback( LPDDPIXELFORMAT pDDPixFmt, LPVOID pContext )
{

// Original Function Definition
//HRESULT CALLBACK D3DEnumPixelFormatsCallback(
//  LPDDPIXELFORMAT lpDDPixFmt,
//  LPVOID          lpContext
//);

// Allocate temporary storage for converted data
TDDPixelFormat* PixelFormat = new TDDPixelFormat( TDx_3D_OnEnumZBufferFormatsOwner );
if (PixelFormat==NULL)
    {
    if (TDx_3D_OnEnumZBufferFormatsOwner->FOnError) TDx_3D_OnEnumZBufferFormatsOwner->FOnError( TDx_3D_OnEnumZBufferFormatsOwner, TDx_3D_OnEnumZBufferFormatsOwner->Name+"::Internal_OnEnumZBufferFormatsCallback()", "TDX_ERROR", "Could not create a temporary TDDPixelFormat component." );
    return D3DENUMRET_CANCEL;
    }

// Translate Parameters before calling Original Function
PixelFormat->ArraySize = 1;
if (PixelFormat->ArraySize==1) 
    CopyMemory( PixelFormat->Internal_DDPIXELFORMAT_Ptr, pDDPixFmt, sizeof(DDPIXELFORMAT) );

void* context = pContext;

// transfer to the Callback Event (if it exists)
bool finished = false;
if (TDx_3D_OnEnumZBufferFormatsOwner->OnEnumZBufferFormats)
    TDx_3D_OnEnumZBufferFormatsOwner->OnEnumZBufferFormats(
        TDx_3D_OnEnumZBufferFormatsOwner,PixelFormat, context, finished );

// cleanup
delete PixelFormat;

// Finished, return FALSE to abort
if (finished) return FALSE;
return TRUE;
}

// --------------------------------------------------------------------------
// Internal Interface Access
// --------------------------------------------------------------------------
LPDIRECT3D7 __fastcall TDx_3D::FGetInternal_LPDIRECT3D7()
{
return fLPDIRECT3D7;
}
// --------------------------------------------------------------------------
LPDIRECT3D7* __fastcall TDx_3D::FGetInternal_LPDIRECT3D7_Ptr()
{
return &fLPDIRECT3D7;
}
// --------------------------------------------------------------------------
void __fastcall TDx_3D::FSetInternal_LPDIRECT3D7( LPDIRECT3D7 pLPDIRECT3D7 )
{
if (!fCreated)
    {
    fLPDIRECT3D7 = pLPDIRECT3D7;
    fCreated = (fLPDIRECT3D7!=NULL);
    }
}
// --------------------------------------------------------------------------
void __fastcall TDx_3D::Internal_LPDIRECT3D7_Update()
{
fCreated = (fLPDIRECT3D7!=NULL);
}
// --------------------------------------------------------------------------
LPDIRECT3D __fastcall TDx_3D::FGetInternal_LPDIRECT3D()
{
return fLPDIRECT3D;
}
// --------------------------------------------------------------------------
LPDIRECT3D* __fastcall TDx_3D::FGetInternal_LPDIRECT3D_Ptr()
{
return &fLPDIRECT3D;
}
// --------------------------------------------------------------------------
void __fastcall TDx_3D::FSetInternal_LPDIRECT3D( LPDIRECT3D pLPDIRECT3D )
{
if (!fCreated)
    {
    fLPDIRECT3D = pLPDIRECT3D;
    fCreated = (fLPDIRECT3D!=NULL);
    }
}
// --------------------------------------------------------------------------
void __fastcall TDx_3D::Internal_LPDIRECT3D_Update()
{
fCreated = (fLPDIRECT3D7!=NULL);
}

// --------------------------------------------------------------------------
