#ifndef TDx_3DH
#define TDx_3DH

// ==========================================================================
// File:         TDx_3D.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TDx_3D Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_3D component.
class TD3DDeviceDesc;
class TDDPixelFormat;
class TDx_Draw;
// --------------------------------------------------------------------------
// external classes used by TDx_3D methods.
class TDx_DrawSurface;
class TDx_3DDevice;
class TD3DVertexBufferDesc;
class TDx_3DVertexBuffer;

// --------------------------------------------------------------------------
// Definition for the OnCreate event.

// Definition for the OnDestroy event.

// Definition for the OnError event.

// --------------------------------------------------------------------------
// Definition for the OnEnumDevices callback event.
typedef	__fastcall void	(__closure*     TDx_EnumDevicesCallback ) 
                        ( TObject*            Sender, 
                          AnsiString  DeviceDescription, 
                          AnsiString  DeviceName, 
                          TD3DDeviceDesc* DeviceDesc, 
                          void*               Context, 
                          bool&               Continue ); 

// Definition for the OnEnumZBufferFormats callback event.
typedef	__fastcall void	(__closure*    TDx_EnumZBufferFormatsCallback ) 
                        ( TObject*            Sender, 
                          TDDPixelFormat* PixelFormat, 
                          void*               Context, 
                          bool&               Continue ); 

// ==========================================================================
// Class:       TDx_3D
// Description: The TDx_3D component wraps the DirectX IDirect3D7 interface 
//              which is the main Direct3D interface. 
//              
//              It can be used to create the TDx_3DDevice and 
//              TDx_3DVertexBuffer interfaces, enumerate 3D devices and their 
//              supported z-buffer formats and evict managed textures from 
//              local and non-local video memory. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_3D : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_3D : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the TDx_3D::Create() 
    //              method after it has successfully created the internal 
    //              LPDIRECT3D7 used within the TDx_3D component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECT3D7 interface. 
    //              
    //              For example, you could retrieve the TDx_3D component's 
    //              capabilities, Create() dependant components, allocate 
    //              memory, etc. 
    //              
    //              When used in conjunction with the TDx_3D::OnDestroy() 
    //              event, you can fully automate and insulate your code on a 
    //              per-component basis. 
    // ----------------------------------------------------------------------

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_3D::Destroy() method, just before the TDx_3D internal 
    //              LPDIRECT3D7 interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the TDx_3D::OnCreate() event. For 
    //              example, assume you have already setup the 
    //              TDx_3D::OnCreate() event to retrieve the component's 
    //              capabilities, Create() a dependant component and allocate 
    //              some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_3D::OnDestroy() event to deallocate memory, Destroy() 
    //              the dependant components and, when required, destroy the 
    //              capabilities component. 
    // ----------------------------------------------------------------------

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_3D component. 
    // ----------------------------------------------------------------------

    // ----------------------------------------------------------------------
    // Callback:    OnEnumDevices()
    // Description: The OnEnumDevices Callback event processes each result 
    //              returned by TDx_3D::EnumDevices() 
    // ----------------------------------------------------------------------
    __property TDx_EnumDevicesCallback OnEnumDevices = {read=FOnEnumDevices, 
    write=FOnEnumDevices, nodefault}; 

    // ----------------------------------------------------------------------
    // Callback:    OnEnumZBufferFormats()
    // Description: The OnEnumZBufferFormats Callback event processes each 
    //              result returned by TDx_3D::OnEnumZBufferFormats() 
    // ----------------------------------------------------------------------
    __property TDx_EnumZBufferFormatsCallback OnEnumZBufferFormats = 
    {read=FOnEnumZBufferFormats, write=FOnEnumZBufferFormats, nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal LPDIRECT3D7 
    //              used in this component has been successfully created, 
    //              otherwise Created is false. 
    //              
    //              To create the internal LPDIRECT3D7, call the 
    //              TDx_3D::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_3D method or fget/fset. eg. 
    //              D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECT3D7 interface used in the TDx_3D 
    //              component and must be called before any methods of the 
    //              TDx_3D component will function. 
    // Params:      pDx_Draw - 
    //                The Dx_Draw parameter ... 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( TDx_Draw* pDx_Draw ); 

    // ----------------------------------------------------------------------
    // Method:      CreateDevice()
    // Description: The TDx_3D::CreateDevice() method will create a 
    //              TDx_3DDevice interface which can then be used to access 
    //              the draw primitive methods. 
    //              
    //              Multiple rendering devices can be created from a single 
    //              TDx_3D component. However, because they share the same 
    //              physical resources, performance will be significantly 
    //              affected. 
    // Params:      pClsId - 
    //                The ClsId parameter identifies the type of device being 
    //                created. 
    //                
    //                The possible values are: 
    //                
    //                IID_IDirect3DTnLHalDevice 
    //                IID_IDirect3DHALDevice 
    //                IID_IDirect3DMMXDevice 
    //                IID_IDirect3DRGBDevice 
    //              pDDS - 
    //                The DDS parameter references the new 3D device's render 
    //                target surface. 
    //                
    //                The surface must have been created with the 
    //                DDSCAPS_3DDEVICE flag set. 
    //              pD3DDevice - 
    //                The D3DDevice property will reference the address of the 
    //                new 3D device if the method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall CreateDevice( REFCLSID pClsId, TDx_DrawSurface* 
    pDDS, TDx_3DDevice* pD3DDevice ); 

    // ----------------------------------------------------------------------
    // Method:      CreateVertexBuffer()
    // Description: The TDx_3D::CreateVertexBuffer() method will create a 
    //              TDx_VertexBuffer interface which can then be used to 
    //              manipulate vertex buffered vertices. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              CLASS_E_NOAGGREGATION   
    //              D3DERR_INVALIDVERTEXFORMAT 
    //              D3DERR_VBUF_CREATE_FAILED 
    //              DDERR_INVALIDOBJECT   
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    // Params:      pVBDesc - 
    //                The VBDesc parameter references the TD3DVertexBufferDesc 
    //                component describing the number and format of the 
    //                vertices the new vertex buffer will accomodate. 
    //              pD3DVertexBuffer - 
    //                The D3DVertexBuffer paremeter references the new vertex 
    //                buffer if the method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall CreateVertexBuffer( TD3DVertexBufferDesc* 
    pVBDesc, TDx_3DVertexBuffer* pD3DVertexBuffer ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECT3D7 interface used in the TDx_3D 
    //              component and should be called when the internal 
    //              interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      EnumDevices()
    // Description: The TDx3D::EnumDevices() method enumerates the currently 
    //              installed TDx_3D compatible device drivers. 
    //              
    //              This method will trigger an TDx_3D::OnEnumDevices() event 
    //              for each device enumerated. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT   
    //              DDERR_INVALIDPARAMS 
    // Params:      pUserArg - 
    //                The UserArg parameter references application defined data 
    //                to be passed to the TDx_3D::EnumDevicesCallback() event 
    //                each time it is called. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall EnumDevices( void* pUserArg ); 

    // ----------------------------------------------------------------------
    // Method:      EnumZBufferFormats()
    // Description: The TDx_3D::EnumZBufferFormats() method will enumerate 
    //              the supported z-buffer formats for a specified device. 
    //              
    //              This method will trigger a TDx_3D::OnEnumZBufferFormats() 
    //              event for each z-buffer format enumerated. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT   
    //              DDERR_INVALIDPARAMS   
    //              DDERR_NOZBUFFERHW 
    //              DDERR_OUTOFMEMORY 
    // Params:      pDevice - 
    //                The Device parameter identifies the device for which 
    //                enumeration of z-buffer formats is to be performed. 
    //              pContext - 
    //                The Context parameter references application defined data 
    //                to be passed to the TDx_3D::EnumZBufferFormatsCallback 
    //                event each time it is called. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall EnumZBufferFormats( REFCLSID pDevice, void* 
    pContext ); 

    // ----------------------------------------------------------------------
    // Method:      EvictManagedTextures()
    // Description: The TDx_3D::EvictManagedTextures() method removes all 
    //              managed textures from memory. 
    //              
    //              Managed textures are those created with the 
    //              DDSCAPS2_TEXTUREMANAGE or DDSCAPS2_D3DTEXTUREMANAGE 
    //              flags. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall EvictManagedTextures(); 

    // ----------------------------------------------------------------------
    // Method:      Internal_EnumDevicesCallback()
    // Description: The TDx_3D::Internal_EnumDevicesCallback() method is used 
    //              internally by the TDx_3DI_Library to redirect a standard 
    //              callback function to a BCB style event. 
    //              
    //              It receives all enumeration callbacks generated by the 
    //              TDx_3D::EnumDevices() method and on each enumeration 
    //              calls the TDx_3D::OnEnumDevices() event, translating 
    //              parameters between DirectX and BCB as required. 
    // Params:      pDeviceDescription - 
    //                The DeviceDescription parameter references a text 
    //                description of the device. 
    //              pDeviceName - 
    //                The DeviceName parameter references the device name. 
    //              pD3DDeviceDesc - 
    //                The D3DDeviceDesc parameter references a TD3DDeviceDesc 
    //                component containing the device capabilities. 
    //              pContext - 
    //                The Context parameter references application defined data 
    //                to be passed to the callback event. 
    // ----------------------------------------------------------------------
    static HRESULT __stdcall Internal_EnumDevicesCallback( LPSTR 
    pDeviceDescription, LPSTR pDeviceName, LPD3DDEVICEDESC7 pD3DDeviceDesc, 
    LPVOID pContext ); 

    // ----------------------------------------------------------------------
    // Method:      Internal_EnumZBufferFormatsCallback()
    // Description: The TDx_3D::Internal_EnumZBufferFormatsCallback() method 
    //              is used internally by the TDx_3DI_Library to redirect a 
    //              standard callback function to a BCB style event. 
    //              
    //              It receives all enumeration callbacks generated by the 
    //              TDx_3D::EnumZBufferFormats() method and on each 
    //              enumeration calls the TDx_3D::OnEnumZBufferFormats() 
    //              event, translating parameters between DirectX and BCB as 
    //              required. 
    // Params:      pDDPixFmt - 
    //                The DDPixFmt parameter references a TDDPixelFormat 
    //                component containing the pixel format of the enumerated 
    //                z-buffer format. 
    //              pContext - 
    //                The Context parameter references application defined data 
    //                to be passed to the callback event. 
    // ----------------------------------------------------------------------
    static HRESULT __stdcall Internal_EnumZBufferFormatsCallback( 
    LPDDPIXELFORMAT pDDPixFmt, LPVOID pContext ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_3D(TComponent* Owner); 
    virtual __fastcall ~TDx_3D(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_3DI_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECT3D7 Internal_LPDIRECT3D7 = { 
    read=FGetInternal_LPDIRECT3D7, write=FSetInternal_LPDIRECT3D7, nodefault 
    }; 
    __property LPDIRECT3D7* Internal_LPDIRECT3D7_Ptr = { 
    read=FGetInternal_LPDIRECT3D7_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECT3D7_Update(); 
    __property LPDIRECT3D Internal_LPDIRECT3D = { 
    read=FGetInternal_LPDIRECT3D, write=FSetInternal_LPDIRECT3D, nodefault }; 
    __property LPDIRECT3D* Internal_LPDIRECT3D_Ptr = { 
    read=FGetInternal_LPDIRECT3D_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECT3D_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECT3D7 __fastcall FGetInternal_LPDIRECT3D7(); 
    void __fastcall FSetInternal_LPDIRECT3D7( LPDIRECT3D7 pLPDIRECT3D7 ); 
    LPDIRECT3D7* __fastcall FGetInternal_LPDIRECT3D7_Ptr(); 
    LPDIRECT3D __fastcall FGetInternal_LPDIRECT3D(); 
    void __fastcall FSetInternal_LPDIRECT3D( LPDIRECT3D pLPDIRECT3D ); 
    LPDIRECT3D* __fastcall FGetInternal_LPDIRECT3D_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECT3D7 fLPDIRECT3D7; 
    LPDIRECT3D fLPDIRECT3D; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

    // ----------------------------------------------------------------------
    // Callback Variables
    // ----------------------------------------------------------------------
    TDx_EnumDevicesCallback FOnEnumDevices;
    TDx_EnumZBufferFormatsCallback FOnEnumZBufferFormats;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
