#ifndef TDx_3DVertexBufferH
#define TDx_3DVertexBufferH

// ==========================================================================
// File:         TDx_3DVertexBuffer.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TDx_3DVertexBuffer Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_3DVertexBuffer component.
class TDx_3D;
// --------------------------------------------------------------------------
// external classes used by TDx_3DVertexBuffer methods.
class TD3DVertexBufferDesc;
class TDx_3DDevice;
class TDx_3DVertexBuffer;
class TD3DDrawPrimitiveStridedData;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDx_3DVertexBuffer
// Description: The TDx_3DVertexBuffer component wraps the DirectX 
//              IDirect3DVertexBuffer7 interface, which is used manipulate a 
//              collection of vertices. 
//              
//              It can be used to optimize, lock and unlock the vertex 
//              buffer,  retrieve its description and process normal or 
//              strided vertices. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_3DVertexBuffer : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_3DVertexBuffer : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_3DVertexBuffer::Create() method after it has 
    //              successfully created the internal LPDIRECT3DVERTEXBUFFER7 
    //              used within the TDx_3DVertexBuffer component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECT3DVERTEXBUFFER7 
    //              interface. 
    //              
    //              For example, you could retrieve the TDx_3DVertexBuffer 
    //              component's capabilities, Create() dependant components, 
    //              allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_3DVertexBuffer::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_3DVertexBuffer::Destroy() method, just before the 
    //              TDx_3DVertexBuffer internal LPDIRECT3DVERTEXBUFFER7 
    //              interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the 
    //              TDx_3DVertexBuffer::OnCreate() event. For example, assume 
    //              you have already setup the TDx_3DVertexBuffer::OnCreate() 
    //              event to retrieve the component's capabilities, Create() 
    //              a dependant component and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_3DVertexBuffer::OnDestroy() event to deallocate 
    //              memory, Destroy() the dependant components and, when 
    //              required, destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_3DVertexBuffer component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECT3DVERTEXBUFFER7 used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECT3DVERTEXBUFFER7, call the 
    //              TDx_3DVertexBuffer::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_3DVertexBuffer method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECT3DVERTEXBUFFER7 interface used in the 
    //              TDx_3DVertexBuffer component and must be called before 
    //              any methods of the TDx_3DVertexBuffer component will 
    //              function. 
    // Params:      pD3D - 
    //                The D3D parameter ... 
    //              pVertexBufferDesc - 
    //                The VertexBufferDesc parameter ... 
    //              pFlags - 
    //                The Flags parameter ... 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( TDx_3D* pD3D, TD3DVertexBufferDesc* 
    pVertexBufferDesc, dword pFlags ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECT3DVERTEXBUFFER7 interface used in the 
    //              TDx_3DVertexBuffer component and should be called when 
    //              the internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      GetVertexBufferDesc()
    // Description: The TDx_3DVertexBuffer::GetVertexBufferDesc() method 
    //              retrieves the vertex buffer's description. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with the following value: 
    //              DDERR_INVALIDPARAMS 
    // Params:      pVBDesc - 
    //                The VBDesc parameter references the TD3DVertexBufferDesc 
    //                component that will hold the vertex buffer's description 
    //                if the method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetVertexBufferDesc( TD3DVertexBufferDesc* 
    pVBDesc ); 

    // ----------------------------------------------------------------------
    // Method:      Lock()
    // Description: The TDx_3DVertexBuffer::Lock() method retrieves a pointer 
    //              to the now locked vertex buffer's memory. 
    //              
    //              While locked, the vertex buffer can be accessed and 
    //              modified as required. 
    //              Attempting to render a locked vertex buffer will merely 
    //              return D3DERR_VERTEXBUFFERLOCKED errors until 
    //              TDx_3DVertexBuffer::Unlock() is called. 
    //              Locking often causes the system to hold Win16Mutex (also 
    //              called Win16Lock), meaning GUI debuggers cannot operate 
    //              during the lock period. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              D3DERR_VERTEXBUFFEROPTIMIZED 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    //              DDERR_SURFACEBUSY 
    //              DDERR_SURFACELOST 
    // Params:      pFlags - 
    //                The Flags parameter indicates the type of lock required. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DDLOCK_DISCARDCONTENTS -
    //                    Allow Direct3D or the device driver to designate 
    //                    alternative memory areas for the vertex buffer if 
    //                    neccessary. 
    //                    Mainly useful when clearing and rewriting the 
    //                    vertex buffer. 
    //                  DDLOCK_NOOVERWRITE -
    //                    No vertices referred to in vertex buffer drawing 
    //                    calls since the start of the last frame or last 
    //                    lock without this flag will be changed during this 
    //                    lock. 
    //                    Mainly useful when appending to the vertex buffer. 
    //                  DDLOCK_NOSYSLOCK -
    //                    Avoid using Win16Mutex (Win16Lock) if possible. 
    //                  DDLOCK_READONLY -
    //                    Vertex buffer data will only be read during this 
    //                    lock. 
    //                  DDLOCK_SURFACEMEMORYPTR -
    //                    A valid vertex buffer memory pointer is retrieved. 
    //                    
    //                    This is the default setting. 
    //                  DDLOCK_WAIT -
    //                    Retry locking until a lock is obtained or an error 
    //                    other than DDERR_SURFACEBUSY occurs. 
    //                  DDLOCK_WRITEONLY -
    //                    Vertex buffer data will only be written during this 
    //                    lock. 
    //              pData - 
    //                The Data parameter will reference a vertex buffer memory 
    //                pointer if the method returns successfully. 
    //              pSize - 
    //                The Size parameter will reference the memory size of the 
    //                vertex buffer if the method returns successfully. 
    //                
    //                Set this parameter to NULL if the buffer size is not 
    //                needed. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Lock( dword pFlags, void** pData, dword* pSize ); 

    // ----------------------------------------------------------------------
    // Method:      Optimize()
    // Description: The TDx_3DVertexBuffer::Optimize() method modifies an 
    //              unlocked vertex buffer for optimal performance according 
    //              to its settings and current system environment. 
    //              
    //              The optimal setup for the vertex buffer is automatically 
    //              adjusted as circumstances dictate. 
    //              As a result, you cannot TDx_3DVertexBuffer::Lock() an 
    //              optimised vertex buffer or use it as the destination 
    //              buffer of the TDx_3DVertexBuffer::ProcessVertices() or 
    //              TDx_3DVertexBuffer::ProcessVerticesStrided() methods. 
    //              Optimised vertex buffers improve performance, but since 
    //              you cannot modify them, they are best used for static 
    //              vertex geometries. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              D3DERR_VERTEXBUFFEROPTIMIZED 
    //              D3DERR_VERTEXBUFFERLOCKED 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    // Params:      pD3DDevice - 
    //                The D3DDevice parameter references the TDx_3DDevice for 
    //                which the vertex buffer is to be optimized. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Optimize( TDx_3DDevice* pD3DDevice ); 

    // ----------------------------------------------------------------------
    // Method:      ProcessVerticies()
    // Description: The TDx_3DVertexBuffer::ProcessVertices() method 
    //              processes or moves vertices from one vertex buffer to 
    //              another. 
    //              
    //              The destination buffer calls this method, effectively 
    //              processing transformed vertices into itself from a source 
    //              vertex buffer. 
    //              Processing consists of applying the transformation 
    //              matrices of the TDx_3DDevice plus optional clipping, 
    //              lighting and extents processing. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              D3DERR_INVALIDVERTEXFORMAT 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    //              DDERR_SURFACEBUSY 
    //              DDERR_SURFACELOST 
    // Params:      pVertexOp - 
    //                The VertexOp parameter indicates how to process vertices 
    //                as they are transferred. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  D3DVOP_CLIP -
    //                    Transform and clip vertices using the viewing 
    //                    frustrum. 
    //                    Cannot be set for vertex buffers without clipping 
    //                    information. 
    //                    eg: those with D3DVBCAPS_DONOTCLIP set. 
    //                  D3DVOP_EXTENTS -
    //                    Transform and update the screen rectangle extents 
    //                    when the vertices are rendered. 
    //                    
    //                    May improve performance, but the 
    //                    TDx_3DDevice::GetClipStatus() extents are not 
    //                    updated when this technique is employed. 
    //                  D3DVOP_LIGHT -
    //                    Transform and light the vertices. 
    //                  D3DVOP_TRANSFORM -
    //                    Transform the vertices using the world, view and 
    //                    projection matrices of the TDx_3DDevice. 
    //                    
    //                    This flag must always be set or the method will 
    //                    fail, returning DDERR_INVALIDPARAMS. 
    //              pDestIndex - 
    //                The DestIndex parameter indicates where processed 
    //                vertices are to be placed in this, the destination 
    //                buffer. 
    //              pCount - 
    //                The Count parameter defines how many of the source buffer 
    //                vertices to process. 
    //              pSrcBuffer - 
    //                The SrcBuffer parameter references the source 
    //                TDx_3DVertexBuffer. 
    //              pSrcIndex - 
    //                The SrcIndex parameter indicates where in the source 
    //                buffer to begin processing. 
    //              pD3DDevice - 
    //                The D3DDevice parameter references the TDx_3DDevice that 
    //                is being used to transform the vertices. 
    //              pFlags - 
    //                The Flags parameter indicates whether unmodified vertices 
    //                should be transferred. 
    //                The described effect applies when the flag is set. 
    //                
    //                Set 0 for default processing, which is still copy 
    //                vertices unmodified by the current vertex operation from 
    //                the source to the destination buffer. 
    //                Flags:
    //                  D3DPV_DONOTCOPYDATA -
    //                    Copy only those vertices modified in the current 
    //                    vertex operation to the destination buffer. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall ProcessVerticies( dword pVertexOp, dword 
    pDestIndex, dword pCount, TDx_3DVertexBuffer* pSrcBuffer, dword 
    pSrcIndex, TDx_3DDevice* pD3DDevice, dword pFlags ); 

    // ----------------------------------------------------------------------
    // Method:      ProcessVerticiesStrided()
    // Description: The TDx_3DVertexBuffer::ProcessVerticesStrided() method 
    //              processes or moves strided vertices from one vertex 
    //              buffer to another. 
    //              
    //              The destination buffer calls this method, effectively 
    //              processing transformed strided vertices into itself from 
    //              a source strided vertex buffer. 
    //              Processing consists of applying the transformation 
    //              matrices of the TDx_3DDevice plus optional clipping, 
    //              lighting and extents processing. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              D3DERR_INVALIDVERTEXFORMAT 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    //              DDERR_SURFACEBUSY 
    //              DDERR_SURFACELOST 
    // Params:      pVertexOp - 
    //                The VertexOp parameter indicates how to process vertices 
    //                as they are transferred. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  D3DVOP_CLIP -
    //                    Transform and clip vertices using the viewing 
    //                    frustrum. 
    //                    Cannot be set for vertex buffers without clipping 
    //                    information. 
    //                    eg: those with D3DVBCAPS_DONOTCLIP set. 
    //                  D3DVOP_EXTENTS -
    //                    Transform and update the screen rectangle extents 
    //                    when the vertices are rendered. 
    //                    
    //                    May improve performance, but the 
    //                    TDx_3DDevice::GetClipStatus() extents are not 
    //                    updated when this technique is employed. 
    //                  D3DVOP_LIGHT -
    //                    Transform and light the vertices. 
    //                  D3DVOP_TRANSFORM -
    //                    Transform the vertices using the world, view and 
    //                    projection matrices of the TDx_3DDevice. 
    //                    
    //                    This flag must always be set or the method will 
    //                    fail, returning DDERR_INVALIDPARAMS. 
    //              pDestIndex - 
    //                The DestIndex parameter indicates where processed 
    //                vertices are to be placed in this, the destination 
    //                buffer. 
    //              pCount - 
    //                The Count parameter defines how many of the source buffer 
    //                vertices to process. 
    //              pVertexArray - 
    //                The VertexArray parameter references a 
    //                TD3DDrawPrimitiveStridedData component describing the 
    //                vertices of the primitive to be processed. 
    //              pSrcIndex - 
    //                The SrcIndex parameter indicates where in the 
    //                TD3DDrawPrimitiveStidedData component to begin processing 
    //                vertices. 
    //              pD3DDevice - 
    //                The D3DDevice parameter references the TDx_3DDevice that 
    //                is being used to transform the vertices. 
    //              pFlags - 
    //                The Flags parameter indicates whether unmodified vertices 
    //                should be transferred. 
    //                The described effect applies when the flag is set. 
    //                
    //                Set 0 for default processing, which is still copy 
    //                vertices unmodified by the current vertex operation from 
    //                the source to the destination buffer. 
    //                Flags:
    //                  DONOTCOPYDATA -
    //                    Copy only those vertices modified in the current 
    //                    vertex operation to the destination buffer. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall ProcessVerticiesStrided( dword pVertexOp, dword 
    pDestIndex, dword pCount, TD3DDrawPrimitiveStridedData* pVertexArray, 
    dword pSrcIndex, TDx_3DDevice* pD3DDevice, dword pFlags ); 

    // ----------------------------------------------------------------------
    // Method:      Unlock()
    // Description: The TDx_3DVertexBuffer::Unlock() method unlocks a 
    //              previously locked vertex buffer. 
    //              
    //              Locked vertex buffers cannot be rendered until they are 
    //              unlocked. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              D3DERR_VERTEXBUFFERUNLOCKFAILED   
    //              DDERR_GENERIC 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_SURFACEBUSY 
    //              DDERR_SURFACELOST 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Unlock(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_3DVertexBuffer(TComponent* Owner); 
    virtual __fastcall ~TDx_3DVertexBuffer(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_3DI_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECT3DVERTEXBUFFER7 Internal_LPDIRECT3DVERTEXBUFFER7 = { 
    read=FGetInternal_LPDIRECT3DVERTEXBUFFER7, 
    write=FSetInternal_LPDIRECT3DVERTEXBUFFER7, nodefault }; 
    __property LPDIRECT3DVERTEXBUFFER7* Internal_LPDIRECT3DVERTEXBUFFER7_Ptr 
    = { read=FGetInternal_LPDIRECT3DVERTEXBUFFER7_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECT3DVERTEXBUFFER7_Update(); 
    __property LPDIRECT3DVERTEXBUFFER Internal_LPDIRECT3DVERTEXBUFFER = { 
    read=FGetInternal_LPDIRECT3DVERTEXBUFFER, 
    write=FSetInternal_LPDIRECT3DVERTEXBUFFER, nodefault }; 
    __property LPDIRECT3DVERTEXBUFFER* Internal_LPDIRECT3DVERTEXBUFFER_Ptr = 
    { read=FGetInternal_LPDIRECT3DVERTEXBUFFER_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECT3DVERTEXBUFFER_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECT3DVERTEXBUFFER7 __fastcall 
    FGetInternal_LPDIRECT3DVERTEXBUFFER7(); 
    void __fastcall FSetInternal_LPDIRECT3DVERTEXBUFFER7( 
    LPDIRECT3DVERTEXBUFFER7 pLPDIRECT3DVERTEXBUFFER7 ); 
    LPDIRECT3DVERTEXBUFFER7* __fastcall 
    FGetInternal_LPDIRECT3DVERTEXBUFFER7_Ptr(); 
    LPDIRECT3DVERTEXBUFFER __fastcall FGetInternal_LPDIRECT3DVERTEXBUFFER(); 
    void __fastcall FSetInternal_LPDIRECT3DVERTEXBUFFER( 
    LPDIRECT3DVERTEXBUFFER pLPDIRECT3DVERTEXBUFFER ); 
    LPDIRECT3DVERTEXBUFFER* __fastcall 
    FGetInternal_LPDIRECT3DVERTEXBUFFER_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECT3DVERTEXBUFFER7 fLPDIRECT3DVERTEXBUFFER7; 
    LPDIRECT3DVERTEXBUFFER fLPDIRECT3DVERTEXBUFFER; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
